@@ -9,14 +9,16 @@ use Business::Shipping;
 my $rate_request = Business::Shipping->rate_request( shipper => 'Offline::UPS' );
 
 $rate_request->submit(
-	service		=> 'XPD',
+	service		=> 'UPSSTD',
 	weight		=> 20,
 	from_zip	=> '98682',
-	from_state	=> 'Washington',
+	from_state	=> 'WA',
 	to_zip		=> 'N2H6S9',
 	to_country	=> 'Canada',
 	event_handlers => {
+		trace 	=> 'STDERR',
 		debug 	=> 'STDERR',
+		#debug3 	=> undef,
 		error	=> 'STDERR',
 	},
 	#download	=> 1,
@@ -29,7 +31,7 @@ print "offline = " . $rate_request->total_charges() . "\n";
 my $rate_request_online = Business::Shipping->rate_request( shipper => 'Online::UPS' );
 
 $rate_request_online->submit(
-	service		=> 'XPD',
+	service		=> 'UPSSTD',
 	weight		=> 20,
 	from_zip	=> '98682',
 	to_zip		=> 'N2H6S9',
